@@ -2,8 +2,8 @@
 
 use crate::column::{ColumnFun, ColumnHeader, ColumnSingle, NamedCol, Writer};
 use crate::comp::{Comp, CompMaker};
-use crate::num::str_to_d_lossy;
-use crate::{err, Error, Result, StringLine, TextLine};
+use crate::text::Text;
+use crate::util::{err, Error, Result, StringLine, TextLine};
 use lazy_static::lazy_static;
 use std::cell::RefCell;
 use std::cmp::Ordering;
@@ -152,7 +152,7 @@ impl AggList {
     /// OutName,Column,Agg,Pattern
     pub fn push_first_prefix(&mut self, spec: &str) -> Result<()> {
         self.v.insert(0, AggCol::new_prefix(spec)?);
-	Ok(())
+        Ok(())
     }
     /// OutName,Column,Agg,Pattern
     pub fn push_append(&mut self, spec: &str) -> Result<()> {
@@ -205,158 +205,153 @@ impl AggList {
 #[derive(Debug)]
 struct Merge {
     // input sub-delimiter
-    delim : u8,
+    delim: u8,
     // output sub-delimiter
-    out_delim : u8,
+    out_delim: u8,
     // output only this many parts
-    max_parts : usize,
+    max_parts: usize,
     // Comparator for Min and Max
     comp: Comp,
     // Sort the parts first
-    do_sort : bool,
+    do_sort: bool,
     // remove adjacent duplicates (according to Comp). Happens after sort.
-    do_uniq : bool,
+    do_uniq: bool,
     // do not write parts shorter than this
-    min_len : usize,
+    min_len: usize,
     // do not write parts longer than this
-    max_len : usize,
+    max_len: usize,
     // write the number of parts, rather than the parts themselves
-    do_count : bool,
+    do_count: bool,
     // internal temp space
-    data : TextLine,
+    data: TextLine,
 }
 
 impl Default for Merge {
     fn default() -> Self {
-	Self {
-	    delim: b',',
-	    out_delim: b',',
-	    max_parts: usize::MAX,
-	    comp: Comp::default(),
-	    do_sort : false,
-	    do_uniq : false,
-	    min_len : 0,
-	    max_len : usize::MAX,
-	    do_count : false,
-	    data : TextLine::default()
-	}
+        Self {
+            delim: b',',
+            out_delim: b',',
+            max_parts: usize::MAX,
+            comp: Comp::default(),
+            do_sort: false,
+            do_uniq: false,
+            min_len: 0,
+            max_len: usize::MAX,
+            do_count: false,
+            data: TextLine::default(),
+        }
     }
 }
 
 impl Merge {
-    fn new(spec : &str) -> Result<Self> {
-	let mut m = Self::default();
-	let mut sp = spec;
-	loop {
-	    if sp.is_empty() {
-		break;
-	    }
-	    if sp.starts_with("comp:") {
-		m.add_one(sp)?;
-		break;
-	    }
-	    if sp.as_bytes()[0] == b'D' {
-		if sp.len() < 3 {
-		    return err!("Merge Delim Spec must be three bytes, e.g. 'D.,'");
-		}
-		m.add_one(&sp[0..3])?;
-		sp = &sp[3..];
-		if !sp.is_empty() {
-		    if sp.as_bytes()[0] != b'.' {
-			return err!("Merge Delim Spec must be three bytes, e.g. 'D.,'");
-		    }
-		    sp = &sp[1..];
-		}
-	    }
-	    if let Some((a, b)) = sp.split_once('.') {
-		m.add_one(a)?;
-		sp = b;
-	    }
-	    else {
-		m.add_one(sp)?;
-		break;
-	    }
-	}
-	Ok(m)
+    fn new(spec: &str) -> Result<Self> {
+        let mut m = Self::default();
+        let mut sp = spec;
+        loop {
+            if sp.is_empty() {
+                break;
+            }
+            if sp.starts_with("comp:") {
+                m.add_one(sp)?;
+                break;
+            }
+            if sp.as_bytes()[0] == b'D' {
+                if sp.len() < 3 {
+                    return err!("Merge Delim Spec must be three bytes, e.g. 'D.,'");
+                }
+                m.add_one(&sp[0..3])?;
+                sp = &sp[3..];
+                if !sp.is_empty() {
+                    if sp.as_bytes()[0] != b'.' {
+                        return err!("Merge Delim Spec must be three bytes, e.g. 'D.,'");
+                    }
+                    sp = &sp[1..];
+                }
+            }
+            if let Some((a, b)) = sp.split_once('.') {
+                m.add_one(a)?;
+                sp = b;
+            } else {
+                m.add_one(sp)?;
+                break;
+            }
+        }
+        Ok(m)
     }
-    fn add_one(&mut self, spec : &str) -> Result<()> {
-	if spec.is_empty() {
-	    return err!("Invalid empty Merge Part");
-	}
-	else if spec.as_bytes()[0] == b'D' {
-	    if spec.len() != 3 {
-		return err!("Merge Delim Spec must be three bytes, e.g. 'D.,'");
-	    }
-	    self.delim = spec.as_bytes()[1];
-	    self.out_delim = spec.as_bytes()[2];
-	}
-	else if spec.eq_ignore_ascii_case("sort") {
-	    self.do_sort = true;
-	}
-	else if spec.eq_ignore_ascii_case("uniq") {
-	    self.do_uniq = true;
-	}
-	else if spec.eq_ignore_ascii_case("count") {
-	    self.do_count = true;
-	}
-	else if let Some(val) = spec.strip_prefix("comp:") {
-	    self.comp = CompMaker::make_comp(val)?;
-	}
-	else if let Some(val) = spec.strip_prefix("min_len:") {
-	    self.min_len = val.parse::<usize>()?;
-	}
-	else if let Some(val) = spec.strip_prefix("max_len:") {
-	    self.max_len = val.parse::<usize>()?;
-	}
-	else if let Some(val) = spec.strip_prefix("max_parts:") {
-	    self.max_parts = val.parse::<usize>()?;
-	}
-	else {
-	    return err!("Unrecognized Merge Part '{}'", spec);
-	}
-	Ok(())
+    fn add_one(&mut self, spec: &str) -> Result<()> {
+        if spec.is_empty() {
+            return err!("Invalid empty Merge Part");
+        } else if spec.as_bytes()[0] == b'D' {
+            if spec.len() != 3 {
+                return err!("Merge Delim Spec must be three bytes, e.g. 'D.,'");
+            }
+            self.delim = spec.as_bytes()[1];
+            self.out_delim = spec.as_bytes()[2];
+        } else if spec.eq_ignore_ascii_case("sort") {
+            self.do_sort = true;
+        } else if spec.eq_ignore_ascii_case("uniq") {
+            self.do_uniq = true;
+        } else if spec.eq_ignore_ascii_case("count") {
+            self.do_count = true;
+        } else if let Some(val) = spec.strip_prefix("comp:") {
+            self.comp = CompMaker::make_comp(val)?;
+        } else if let Some(val) = spec.strip_prefix("min_len:") {
+            self.min_len = val.parse::<usize>()?;
+        } else if let Some(val) = spec.strip_prefix("max_len:") {
+            self.max_len = val.parse::<usize>()?;
+        } else if let Some(val) = spec.strip_prefix("max_parts:") {
+            self.max_parts = val.parse::<usize>()?;
+        } else {
+            return err!("Unrecognized Merge Part '{}'", spec);
+        }
+        Ok(())
     }
 }
 
 impl Agg for Merge {
     fn add(&mut self, data: &[u8]) {
-	if !data.is_empty() {
-	    if !self.data.line.is_empty() {
-		self.data.line.push(self.delim);
-	    }
-	    self.data.line.extend_from_slice(data);
-	}
+        if !data.is_empty() {
+            if !self.data.line.is_empty() {
+                self.data.line.push(self.delim);
+            }
+            self.data.line.extend_from_slice(data);
+        }
     }
     fn result(&mut self, w: &mut dyn Write) -> Result<()> {
-	self.data.split(self.delim);
-	if self.do_sort {
-	    self.data.parts.sort_by(|a, b| self.comp.comp(a.get(&self.data.line), b.get(&self.data.line)));
-	}
-	if self.do_uniq {
-            self.data.parts.dedup_by(|a, b| self.comp.equal(a.get(&self.data.line), b.get(&self.data.line)));
-	}
-	if self.do_count {
-	    write!(w, "{}", self.data.parts.len())?;
-	}
-	else {
-	    let mut num_written = 0;
-	    for x in &self.data.parts {
-		if x.len() >= self.min_len && x.len() <= self.max_len {
-		    if num_written > 0 {
-			w.write_all(&[self.out_delim])?;
-		    }
-		    w.write_all(x.get(&self.data.line))?;
-		    num_written += 1;
-		    if num_written >= self.max_parts {
-			break;
-		    }
-		}
-	    }
-	}
-	Ok(())
+        self.data.split(self.delim);
+        if self.do_sort {
+            self.data.parts.sort_by(|a, b| {
+                self.comp
+                    .comp(a.get(&self.data.line), b.get(&self.data.line))
+            });
+        }
+        if self.do_uniq {
+            self.data.parts.dedup_by(|a, b| {
+                self.comp
+                    .equal(a.get(&self.data.line), b.get(&self.data.line))
+            });
+        }
+        if self.do_count {
+            write!(w, "{}", self.data.parts.len())?;
+        } else {
+            let mut num_written = 0;
+            for x in &self.data.parts {
+                if x.len() >= self.min_len && x.len() <= self.max_len {
+                    if num_written > 0 {
+                        w.write_all(&[self.out_delim])?;
+                    }
+                    w.write_all(x.get(&self.data.line))?;
+                    num_written += 1;
+                    if num_written >= self.max_parts {
+                        break;
+                    }
+                }
+            }
+        }
+        Ok(())
     }
     fn reset(&mut self) {
-	self.data.line.clear();
+        self.data.line.clear();
     }
 }
 
@@ -415,7 +410,7 @@ impl Mean {
 
 impl Agg for Mean {
     fn add(&mut self, data: &[u8]) {
-        self.val += str_to_d_lossy(data);
+        self.val += data.to_f64_lossy();
         self.cnt += 1.0;
     }
     fn result(&mut self, w: &mut dyn Write) -> Result<()> {
@@ -444,7 +439,7 @@ impl Sum {
 
 impl Agg for Sum {
     fn add(&mut self, data: &[u8]) {
-        self.val += str_to_d_lossy(data);
+        self.val += data.to_f64_lossy();
     }
     fn result(&mut self, w: &mut dyn Write) -> Result<()> {
         write!(w, "{}", self.val)?;
@@ -492,37 +487,41 @@ struct Prefix {
 
 impl Prefix {
     fn new(spec: &str) -> Result<Self> {
-	if spec.is_empty() {
-	    Ok(Self{val : Vec::new(), empty : true})
-	}
-	else {
-	    err!("Unexpected non-empty pattern passed to Prefix agregator : '{}'", spec)
-	}
+        if spec.is_empty() {
+            Ok(Self {
+                val: Vec::new(),
+                empty: true,
+            })
+        } else {
+            err!(
+                "Unexpected non-empty pattern passed to Prefix agregator : '{}'",
+                spec
+            )
+        }
     }
 }
 
-fn common_prefix(val : &mut Vec<u8>, data : &[u8]) {
+fn common_prefix(val: &mut Vec<u8>, data: &[u8]) {
     if val.len() > data.len() {
-	val.truncate(data.len());
+        val.truncate(data.len());
     }
     #[allow(clippy::needless_range_loop)] // cleaner this way
     for x in 0..val.len() {
-	if val[x] != data[x] {
-	    val.truncate(x);		    
-	    break;
-	}
+        if val[x] != data[x] {
+            val.truncate(x);
+            break;
+        }
     }
 }
 
 impl Agg for Prefix {
     fn add(&mut self, data: &[u8]) {
-	if self.empty {
-	    self.empty = false;
+        if self.empty {
+            self.empty = false;
             self.val.extend_from_slice(data);
-	}
-	else {
-	    common_prefix(&mut self.val, data);
-	}
+        } else {
+            common_prefix(&mut self.val, data);
+        }
     }
     fn result(&mut self, w: &mut dyn Write) -> Result<()> {
         w.write_all(&self.val)?;
@@ -530,7 +529,7 @@ impl Agg for Prefix {
     }
     fn reset(&mut self) {
         self.val.clear();
-	self.empty = true;
+        self.empty = true;
     }
 }
 
@@ -541,39 +540,42 @@ struct Suffix {
 
 impl Suffix {
     fn new(spec: &str) -> Result<Self> {
-	if spec.is_empty() {
-	    Ok(Self{val : Vec::new(), empty : true})
-	}
-	else {
-	    err!("Unexpected non-empty pattern passed to Sufffix agregator : '{}'", spec)
-	}
+        if spec.is_empty() {
+            Ok(Self {
+                val: Vec::new(),
+                empty: true,
+            })
+        } else {
+            err!(
+                "Unexpected non-empty pattern passed to Sufffix agregator : '{}'",
+                spec
+            )
+        }
     }
 }
 
-fn common_suffix(val : &mut Vec<u8>, data : &[u8]) {
+fn common_suffix(val: &mut Vec<u8>, data: &[u8]) {
     let mut ok_bytes = 0;
     for x in val.iter().rev().zip(data.iter().rev()) {
-	if x.0 == x.1 {
-	    ok_bytes += 1;
-	}
-	else {
-	    break;
-	}
+        if x.0 == x.1 {
+            ok_bytes += 1;
+        } else {
+            break;
+        }
     }
     if ok_bytes < val.len() {
-	val.drain(0..(val.len()-ok_bytes));
+        val.drain(0..(val.len() - ok_bytes));
     }
 }
 
 impl Agg for Suffix {
     fn add(&mut self, data: &[u8]) {
-	if self.empty {
-	    self.empty = false;
+        if self.empty {
+            self.empty = false;
             self.val.extend_from_slice(data);
-	}
-	else {
-	    common_suffix(&mut self.val, data);
-	}
+        } else {
+            common_suffix(&mut self.val, data);
+        }
     }
     fn result(&mut self, w: &mut dyn Write) -> Result<()> {
         w.write_all(&self.val)?;
@@ -581,31 +583,30 @@ impl Agg for Suffix {
     }
     fn reset(&mut self) {
         self.val.clear();
-	self.empty = true;
+        self.empty = true;
     }
 }
 
 struct Count {
-    val: usize
+    val: usize,
 }
 
 impl Count {
     fn new(spec: &str) -> Result<Self> {
-	if spec.is_empty() {
-	    Ok(Self{val:0})
-	}
-	else {
-	    err!("Unexpected pattern passed to Count aggregator : '{}'", spec)
-	}
+        if spec.is_empty() {
+            Ok(Self { val: 0 })
+        } else {
+            err!("Unexpected pattern passed to Count aggregator : '{}'", spec)
+        }
     }
 }
 
 impl Agg for Count {
     fn add(&mut self, _data: &[u8]) {
-	self.val += 1;
+        self.val += 1;
     }
     fn result(&mut self, w: &mut dyn Write) -> Result<()> {
-	write!(w, "{}", self.val)?;
+        write!(w, "{}", self.val)?;
         Ok(())
     }
     fn reset(&mut self) {
@@ -647,16 +648,12 @@ impl AggMaker {
         Self::do_add_alias("min", "minimum")?;
         Self::do_add_alias("max", "maximum")?;
         Self::do_add_alias("mean", "avg")?;
-        Self::do_push(
-            "merge",
-            "Merge into a delimited list of values",
-            |p| Ok(Rc::new(RefCell::new(Merge::new(p)?))),
-        )?;
-        Self::do_push(
-            "count",
-            "The number of things aggregated",
-            |p| Ok(Rc::new(RefCell::new(Count::new(p)?))),
-        )?;
+        Self::do_push("merge", "Merge into a delimited list of values", |p| {
+            Ok(Rc::new(RefCell::new(Merge::new(p)?)))
+        })?;
+        Self::do_push("count", "The number of things aggregated", |p| {
+            Ok(Rc::new(RefCell::new(Count::new(p)?)))
+        })?;
         Self::do_push(
             "prefix",
             "The longest common prefix of the input values",
@@ -755,17 +752,17 @@ impl AggMaker {
         for x in &*mm {
             println!("{:12}{}", x.tag, x.help);
         }
-	println!();
-	println!("'merge' pattern is a period delimited set of any of the following :");
-	println!("sort        --  sort the parts");
-	println!("uniq        -- Remove any adjacent equal parts. Happens after sorting.");
-	println!("count       -- Display the count of parts, not the parts themselves.");
-	println!("max_len:N   -- Discard any parts longer than N bytes.");
-	println!("min_len:N   -- Discard any parts shorter than N bytes.");
-	println!("max_parts:N -- Display only the first N parts.");
-	println!("Dxy         -- 'x' is the input delimiter and 'y' is the output delimiter. Default is comma for both.");
-	println!("comp:spec   -- Spec for comparator for sort or uniq. Must be last piece.");
-	println!();
+        println!();
+        println!("'merge' pattern is a period delimited set of any of the following :");
+        println!("sort        --  sort the parts");
+        println!("uniq        -- Remove any adjacent equal parts. Happens after sorting.");
+        println!("count       -- Display the count of parts, not the parts themselves.");
+        println!("max_len:N   -- Discard any parts longer than N bytes.");
+        println!("min_len:N   -- Discard any parts shorter than N bytes.");
+        println!("max_parts:N -- Display only the first N parts.");
+        println!("Dxy         -- 'x' is the input delimiter and 'y' is the output delimiter. Default is comma for both.");
+        println!("comp:spec   -- Spec for comparator for sort or uniq. Must be last piece.");
+        println!();
         println!("See also https://avjewe.github.io/cdxdoc/Aggregator.html.");
     }
     /// Create a Agg from a name and a pattern
@@ -803,34 +800,34 @@ mod tests {
     use super::*;
     #[test]
     fn suffix() {
-	let mut v = b"123456789".to_vec();
-	common_suffix(&mut v,  b"123456789");
-	assert_eq!(&v, b"123456789");
-	common_suffix(&mut v,  b"23456789");
-	assert_eq!(&v, b"23456789");
-	common_suffix(&mut v,  b"dsjfldkjsfaslkjfaslkfjadflkj3456789");
-	assert_eq!(&v, b"3456789");
-	common_suffix(&mut v,  b"");
-	assert_eq!(&v, b"");
-	common_suffix(&mut v,  b"");
-	assert_eq!(&v, b"");
-	common_suffix(&mut v,  b"sadfddsafasdgfg");
-	assert_eq!(&v, b"");
+        let mut v = b"123456789".to_vec();
+        common_suffix(&mut v, b"123456789");
+        assert_eq!(&v, b"123456789");
+        common_suffix(&mut v, b"23456789");
+        assert_eq!(&v, b"23456789");
+        common_suffix(&mut v, b"dsjfldkjsfaslkjfaslkfjadflkj3456789");
+        assert_eq!(&v, b"3456789");
+        common_suffix(&mut v, b"");
+        assert_eq!(&v, b"");
+        common_suffix(&mut v, b"");
+        assert_eq!(&v, b"");
+        common_suffix(&mut v, b"sadfddsafasdgfg");
+        assert_eq!(&v, b"");
     }
     #[test]
     fn prefix() {
-	let mut v = b"123456789".to_vec();
-	common_prefix(&mut v,  b"123456789");
-	assert_eq!(&v, b"123456789");	
-	common_prefix(&mut v,  b"12345678");
-	assert_eq!(&v, b"12345678");	
-	common_prefix(&mut v,  b"1234567sdfhasflhasflasflaksjfasdkhj");
-	assert_eq!(&v, b"1234567");	
-	common_prefix(&mut v,  b"sdkjadflkjafdakjdsf");
-	assert_eq!(&v, b"");	
-	common_prefix(&mut v,  b"sdkjadflkjafdakjdsf");
-	assert_eq!(&v, b"");	
-	common_prefix(&mut v,  b"");
-	assert_eq!(&v, b"");	
+        let mut v = b"123456789".to_vec();
+        common_prefix(&mut v, b"123456789");
+        assert_eq!(&v, b"123456789");
+        common_prefix(&mut v, b"12345678");
+        assert_eq!(&v, b"12345678");
+        common_prefix(&mut v, b"1234567sdfhasflhasflasflaksjfasdkhj");
+        assert_eq!(&v, b"1234567");
+        common_prefix(&mut v, b"sdkjadflkjafdakjdsf");
+        assert_eq!(&v, b"");
+        common_prefix(&mut v, b"sdkjadflkjafdakjdsf");
+        assert_eq!(&v, b"");
+        common_prefix(&mut v, b"");
+        assert_eq!(&v, b"");
     }
 }
