@@ -32,7 +32,9 @@ pub const MAINLIST: &[OneMain] = &[
     proc! {"cut",  "Select colums.",  cut_main::main},
     proc! {"join",  "Join files on a matching column",  join_main::main},
     proc! {"sort", "Sort files.", sort_main::main},
+    proc! {"tabs", "Display files.", tabs_main::main},
     proc! {"tooltest", "Test command line tools.", tooltest_main::main},
+    proc! {"transpose", "Transpose a file.", transpose_main::main},
     proc! {"uniq", "Merge duplicate lines.", uniq_main::main},
     proc! {"verify", "Verify the contents of a file.", verify_main::main},
 ];
