@@ -10,7 +10,9 @@ mod cgrep_main;
 mod cut_main;
 mod join_main;
 mod sort_main;
+mod tabs_main;
 mod tooltest_main;
+mod transpose_main;
 mod uniq_main;
 mod verify_main;
 
