@@ -15,6 +15,7 @@ use std::os::unix::ffi::OsStrExt;
 use std::path::Path;
 use std::str::FromStr;
 use std::{fmt, str};
+use tokio_stream::StreamExt;
 
 /// Shorthand for returning an error Result
 #[macro_export]
@@ -38,6 +39,7 @@ macro_rules! err_type {
 /// Various errors
 #[derive(Debug)]
 #[non_exhaustive]
+#[allow(clippy::large_enum_variant)]
 pub enum Error {
     /// Custom cdx error
     Error(String),
@@ -53,6 +55,12 @@ pub enum Error {
     FromUtf8Error(std::string::FromUtf8Error),
     /// pass through Utf8Error
     Utf8Error(std::str::Utf8Error),
+    /// pass through DecodeError
+    Base64(base64::DecodeError),
+    /// pass through SdkError
+    MyGetObjectError(aws_sdk_s3::SdkError<aws_sdk_s3::error::GetObjectError>),
+    /// bytestream
+    ByteStreamError(aws_smithy_http::byte_stream::Error),
     /// common error with long default string
     NeedLookup,
     /// be an error, but don't report anything
@@ -76,12 +84,18 @@ impl Error {
     }
 }
 
+err_type!(base64::DecodeError, Error::Base64);
 err_type!(std::str::Utf8Error, Error::Utf8Error);
 err_type!(regex::Error, Error::RegexError);
 err_type!(std::string::FromUtf8Error, Error::FromUtf8Error);
 err_type!(std::io::Error, Error::IoError);
 err_type!(std::num::ParseIntError, Error::ParseIntError);
 err_type!(std::num::ParseFloatError, Error::ParseFloatError);
+err_type!(aws_smithy_http::byte_stream::Error, Error::ByteStreamError);
+err_type!(
+    aws_sdk_s3::SdkError<aws_sdk_s3::error::GetObjectError>,
+    Error::MyGetObjectError
+);
 
 impl fmt::Display for Error {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -93,6 +107,9 @@ impl fmt::Display for Error {
             Error::RegexError(s) => write!(f, "RegexError : {}", s)?,
             Error::FromUtf8Error(s) => write!(f, "FromUtf8Error : {}", s)?,
             Error::Utf8Error(s) => write!(f, "Utf8Error : {}", s)?,
+            Error::MyGetObjectError(s) => write!(f, "GetObjectError : {}", s)?,
+            Error::ByteStreamError(s) => write!(f, "ByteStreamError : {}", s)?,
+            Error::Base64(s) => write!(f, "Base64 : {}", s)?,
             Error::NeedLookup => write!(
                 f,
                 "ColumnSet.lookup() must be called before ColumnSet.select()"
@@ -442,6 +459,77 @@ impl<'a> Iterator for StringLineIter<'a> {
     }
 }
 
+struct S3Reader {
+    //    name : String,
+    rt: tokio::runtime::Runtime,
+    //    client : aws_sdk_s3::Client,
+    f: aws_sdk_s3::output::GetObjectOutput,
+    left: Option<bytes::Bytes>,
+}
+impl S3Reader {
+    fn new(bucket: &str, key: &str) -> Result<Self> {
+        let rt = tokio::runtime::Builder::new_current_thread()
+            .enable_all()
+            .build()?;
+        let shared_config = rt.block_on(aws_config::load_from_env());
+        let client = aws_sdk_s3::Client::new(&shared_config);
+        let obj = rt.block_on(client.get_object().bucket(bucket).key(key).send())?;
+        Ok(Self {
+            //	    name : key.to_string(),
+            rt,
+            //	    client,
+            f: obj,
+            left: None,
+        })
+    }
+    fn new_path(spec: &str) -> Result<Self> {
+        if let Some(name) = spec.strip_prefix("s3://") {
+            if let Some((a, b)) = name.split_once('/') {
+                Self::new(a, b)
+            } else {
+                err!("Not an S3 file spec '{}'", spec)
+            }
+        } else {
+            err!("Not an S3 file '{}'", spec)
+        }
+    }
+}
+impl Read for S3Reader {
+    fn read(&mut self, buf: &mut [u8]) -> std::result::Result<usize, std::io::Error> {
+        if let Some(bytes) = &self.left {
+            if bytes.len() > buf.len() {
+                buf.clone_from_slice(&bytes[..buf.len()]);
+                self.left = Some(bytes.slice(buf.len()..));
+                return Ok(buf.len());
+            } else {
+                let len = bytes.len();
+                buf[0..len].clone_from_slice(bytes);
+                self.left = None;
+                return Ok(len);
+            }
+        }
+        let bytes_res = self.rt.block_on(self.f.body.try_next());
+        if bytes_res.is_err() {
+            return Err(std::io::Error::new(std::io::ErrorKind::Other, "oh no"));
+        }
+        self.left = bytes_res.unwrap();
+        if let Some(bytes) = &self.left {
+            if bytes.len() > buf.len() {
+                buf.clone_from_slice(&bytes[..buf.len()]);
+                self.left = Some(bytes.slice(buf.len()..));
+                Ok(buf.len())
+            } else {
+                let len = bytes.len();
+                buf[0..bytes.len()].clone_from_slice(bytes);
+                self.left = None;
+                Ok(len)
+            }
+        } else {
+            Ok(0)
+        }
+    }
+}
+
 /// Input file. Wrapped in a type so I can 'impl Debug'
 pub struct Infile(
     /// The file being read
@@ -557,8 +645,8 @@ pub fn get_reader2<P: AsRef<Path>>(name: P) -> Result<Infile> {
         if name == OsStr::new("-") {
             //	    unsafe { Box::new(std::fs::File::from_raw_fd(1)) }
             Box::new(io::stdin())
-            //            } else if name.starts_with("s3://") {
-            //                Box::new(open_s3_file(name)?)
+        //        } else if name.starts_with("s3://") {
+        //            Box::new(S3Reader::new_path(name)?)
         } else if name.as_bytes().starts_with(b"<<") {
             Box::new(std::io::Cursor::new(unescape_vec(&name.as_bytes()[2..])))
         } else {
@@ -579,8 +667,8 @@ pub fn get_reader(name: &str) -> Result<Infile> {
         if name == "-" {
             //	    unsafe { Box::new(std::fs::File::from_raw_fd(1)) }
             Box::new(io::stdin())
-            //            } else if name.starts_with("s3://") {
-            //                Box::new(open_s3_file(name)?)
+        } else if name.starts_with("s3://") {
+            Box::new(S3Reader::new_path(name)?)
         } else if let Some(stripped) = name.strip_prefix("<<") {
             Box::new(std::io::Cursor::new(unescape_vec(stripped.as_bytes())))
         } else {
